import React, { Component } from 'react';
import { Switch, Route } from 'react-router-dom'
import PageFileManager from './components/PageFileManager/PageFileManager'
import PageBatchOperations from './components/PageBatchOperations/PageBatchOperations'
import PageSingleFileOps from './components/PageSingleFileOps/PageSingleFileOps'
import SingleFileOperation from './components/SingleFileOperation/SingleFileOperation'
import FileSelect from './components/Files/FileSelect'

// The Main component renders one of the three provided
// Routes (provided that one matches). Both the /roster
// and /schedule routes will match any pathname that starts
// with /roster or /schedule. The / route will only match
// when the pathname is exactly the string "/"

class Main extends Component {

  constructor(props) {
    super(props);
    this.state = { type: "", filename: "" };
    this.myCallback = this.myCallback.bind(this);
    this.getParentCurrentFile = this.getParentCurrentFile.bind(this);
  }

  getParentCurrentFile = () => this.state.filename;

  myCallback = (data) => {
    if (data !== undefined) {
      console.log("Callback\n", data.label);
      let selected = data.label ? data.label : "no file selected."
      this.setState({ filename: selected });
    }
  }

  render() {
    return (
      <div>
        <FileSelect callbackFromParent={this.myCallback} />
        <p>FILE: {this.state.filename}</p>
        <Switch>
          <Route path='/batch'
            component={PageBatchOperations} />

          <Route path='/single'
            component={PageSingleFileOps} />

          <Route path='/files'
            component={PageFileManager} />

          <Route path="/dtwstructure" render={() =>
            <SingleFileOperation type="dtw"
              filename={this.state.filename}
              getCurrentFile={this.getParentCurrentFile} />} />

          <Route path="/chordify" render={() =>
            <SingleFileOperation type="chordify"
              filename={this.state.filename}
              getCurrentFile={this.getParentCurrentFile} />} />

          <Route path="/score" render={() =>
            <SingleFileOperation type="score"
              filename={this.state.filename}
              getCurrentFile={this.getParentCurrentFile} />} />

          <Route path="/stats" render={() =>
            <SingleFileOperation type="stats"
              filename={this.state.filename}
              getCurrentFile={this.getParentCurrentFile} />} />

        </Switch>
      </div>
    );
  }
}

export default Main;

